#!/usr/bin/php
<?php
define('PATH','/usr/local/esia/public_html');
define('NAME',"check_application.php");
include_once PATH.'/esia.conf.php';
//include_once PATH.'/Model_esia.php';

$ERRORS= array('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3);

function print_usage(){
	echo 'USAGE : '.NAME." -w <widget_id> -u <user_id>\n";
}

function print_help()
{
	print_usage();
	echo "\t".NAME." : Vérifie l'état d'une chaine applicative\n";
	echo "\t\t[-w] 	| --widget		id de la widget de la chaine applicative\n";
	echo "\t\t[-u] 	| --user		id de l'utilisateur qui crée le noeud\n";
	echo "\t\t[-h] 	| --help		affiche l'aide\n";
}

//Définition des paramètres d'exécution
$longopts  = array(
    "widget:",
    "user:",
    "help");
$options = getopt("w:u:h",$longopts); 

$priorities = array(
                $ERRORS['CRITICAL'],
                $ERRORS['WARNING'],
                $ERRORS['UNKNOWN'],
                $ERRORS['OK']);

function priority_state($priorities,$a,$b)
{
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach ($priorities as $v)
		{
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

if(isset($options['h']) || isset($options['help']))
{
	print_help();
	exit($ERRORS['UNKNOWN']);
}

//connection à la base de donnée
try
{
	$dbconnection = new PDO('pgsql:host='.DB_HOST.';port='.DB_PORT.';dbname='.DB_NAME, DB_USERNAME, DB_PWD);
	$dbconnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
}
catch(Exception $e)
{
	echo 'PDO SQL ERROR: '.$e->getMessage().'<br />';
	echo 'N° : '.$e->getCode();
}


if((isset($options['w']) && !empty($options['w'])) && (isset($options['u']) && !empty($options['u'])))
{
	$widget = $options['w'];
	$user = $options['u'];
	$dashboard = getWidget($dbconnection,$widget,$user);
	$dashboard = $dashboard[0];
	$params=$dashboard['option'];

	$i=0;
	$allGlobalState = array();
	$widgetGlobalState = "EventOK";

	$str_widget_global="";
	$str_ret = "";

	foreach($params['link'] as $element)
	{
		if($element['type'] == "node")
		{
			if(!empty($element['id']))
			{
				$state =  countNodeState($dbconnection,$element['id']);
			}
		}
		else if($element['type'] == "groupNode")
		{
			if(!empty($element['id']))
			{
				//On récupère l'état des noeuds dans le parent et l'état des sous-groupes. 
				$state_parent =  getGroupNodeStates($dbconnection,$user,$element['id'],false);
				$state_child =  getGroupNodeStates($dbconnection,$user,$element['id'],true);
				reset($state_parent);
				$state['ok'] = $state_parent[key($state_parent)]['ok'];
				$state['warning'] = $state_parent[key($state_parent)]['warning'];
				$state['critical'] = $state_parent[key($state_parent)]['critical'];
				$state['unknown'] = $state_parent[key($state_parent)]['unknown'];
				//On additionne les 2 tableau pour avoir l'état global d'un groupe
				foreach($state_child as $current)
				{
					$state['ok'] += $current['ok'];
					$state['warning'] += $current['warning'];
					$state['critical'] += $current['critical'];
					$state['unknown'] += $current['unknown'];
				}
			}
		}
		else //service
		{
			if(!empty($element['id']))
			{
				$state =  countServiceAllState($dbconnection,$user,array($element['id']));
			}
		}

		$globalState = "OK";
		if($state["critical"] > 0) {
			$globalState = "CRITICAL";
		}else if ($state["warning"] > 0) {
			$globalState = "WARNING";
		}else if ($state["unknown"] > 0) {
			$globalState = "UNKNOWN";
		}

        if($element['impact'])
        {
                foreach($element['impact'] as $impact) 
                {
                        if($impact == "critical" && $state['critical'] > 0) {
                                array_push($allGlobalState, "critical");
                        }else if ($impact == "warning" && $state['warning'] > 0) {
                                array_push($allGlobalState, "warning");
                        }else if ($impact == "unknown" && $state['unknown'] > 0) {
                                array_push($allGlobalState, "unknown");
                        }
                }
        }

        if($element['type']=='service') $service_info = getServices($dbconnection,null,$element['id']);
        elseif($element['type']=='node') $service_info = getServices($dbconnection,$element['id']);
        elseif($element['type']=='groupNode') $groupName = getGroupName($dbconnection,$element['id']);

		if($element['type']=='service')
		{
			$node_id=$service_info[0]['device_id'];
			$str_ret .= $globalState." : ".$element['titre']." (Service ".$service_info[0]['check_plugin_name']." on node : ".$service_info[0]['device_name'].")\n";
		}
		elseif($element['type']=='node')
		{
			$node_id=$element['id'];
			$str_ret .= $globalState." : ".$element['titre']." (Node ".$service_info[0]['device_name'].")\n";
		}
		else //groupNode
		{
			$str_ret .= $globalState." : ".$element['titre']." (Group ".$groupName.")\n";
		}
		
		$params["link"][$i]['state'] = $globalState;
		$i++;
	}

	$global_status = "OK";
	if(in_array('critical', $allGlobalState)) {
		$global_status = "CRITICAL";
		$return=$ERRORS['CRITICAL'];
	}else if (in_array('warning', $allGlobalState)) {
		$global_status = "WARNING";
		$return=$ERRORS['WARNING'];
	}else if (in_array('unknown', $allGlobalState)) {
		$global_status = "UNKNOWN";
		$return=$ERRORS['UNKNOWN'];
	}

	$str_widget_global ="GLOBAL STATE : $global_status\n";
}
else{
	echo "UNKNOWN - Missing required plugin options\n";
	exit(3);
}

echo "$str_widget_global$str_ret";
exit($return);

function executeStatement(PDOStatement $query,$param=NULL)
{
	$ret = $query->execute($param);
	if($ret===false)
	{
		$err = $query->errorInfo();
		throw new EsiaDbException($err[2]);
	}
	return $query;
}

function getWidget($dbconnection,$widgetid=null,$user=null)
{
	$sql ="SELECT widget_id, widget_name, widget_view_during, widget_position,widget_tile_id,
  				 widget_option, widget_description,widget_id,view_file_path,view_default_option,view_name
		   FROM dashboard_widget
		   JOIN dashboard_view ON widget_view_id = view_id";
	$param=array();
	if(isset($widgetid) && is_numeric($widgetid))
	{
		$sql .= " WHERE widget_id = :widget ";
		$param=array('widget' => $widgetid);
	}
	$sql.=" ORDER BY widget_position;";
	$query= $dbconnection->prepare($sql);
	$query = executeStatement($query,$param );
	$list=array();
	while($row=$query->fetch(PDO::FETCH_OBJ))
	{
		$path = preg_replace('#\:widgetid#', $row->widget_id, $row->view_file_path);
		$list[] = array('id'=>$row->widget_id,
				'name' => $row->widget_name,
				'during' => $row->widget_view_during,
				'position'=>$row->widget_position,
				'option'=>json_decode($row->widget_option,true),
				'default_option'=>json_decode($row->view_default_option,true),
				'description'=>$row->widget_description,
				'path'=>$path,
				'tile'=>$row->widget_tile_id,
				'view_name'=>$row->view_name
		);
	}
	return $list;
}

function countNodeState($dbconnection,$ids)
{
	$list=array();
	$list['OK']=0;
	$list["warning"]=0;
	$list["critical"]=0;
	$list["unknown"]=0;
	$list["pending"]=0;
	$params=array();
	if(is_numeric($ids))
	{
		$query = $dbconnection->prepare("SELECT error_type,count(*)as nbr_error
				FROM esia_error_happen
				JOIN esia_errors ON error_id = happen_error_id
				WHERE happen_device_id=:id AND happen_acquitte=0
				GROUP BY error_type
				UNION 
				SELECT 'OK', count(DISTINCT service_id) as service_total
				FROM esia_services
				WHERE service_previous_state=0 AND service_delete=FALSE AND service_device_id=:id
				UNION
				SELECT 'pending',count(*) as nbr_error
				FROM esia_services
				WHERE service_device_id=:id AND service_last_stdout IS NULL AND service_previous_state=4;");
		$params['id']=$ids;
	}
	elseif (is_array($ids))
	{
		foreach($ids as $id)
		{
			if(!is_numeric($id))
				throw new EsiaDbException('$service array must be numeric');
		}
		$arrids = implode(',', $ids);
		$query = $dbconnection->prepare("SELECT error_type,count(*)as nbr_error
				FROM esia_error_happen
				JOIN esia_errors ON error_id = happen_error_id
				WHERE happen_device_id IN ($arrids) AND happen_acquitte=0
				GROUP BY error_type
				UNION
				SELECT 'OK', count(DISTINCT service_id) as service_total
				FROM esia_services
				WHERE service_previous_state=0 AND service_delete=FALSE AND service_device_id IN ($arrids)
				UNION
				SELECT 'pending',count(*) as nbr_error
				FROM esia_services
				WHERE service_device_id IN ($arrids) AND service_last_stdout IS NULL AND service_previous_state=4;");
	}
	else
	{
		$query = $dbconnection->prepare("SELECT error_type,count(*)as nbr_error
				FROM esia_device
				JOIN esia_error_happen ON device_id = happen_device_id
				JOIN esia_errors ON error_id = happen_error_id
				WHERE device_name=:name AND happen_acquitte=0
				UNION 
				SELECT 'OK', count(DISTINCT service_id) as service_total
				FROM esia_services
				JOIN esia_device ON device_id = service_device_id
				WHERE service_previous_state=0 AND service_delete=FALSE AND device_name=:name
				GROUP BY error_type
				UNION
				SELECT 'pending', count(*) as nbr_error
				FROM esia_services
				JOIN esia_device ON device_id = service_device_id
				WHERE device_name=:name AND service_last_stdout IS NULL AND service_previous_state=4");
		$params['name']=$ids;
	}
	$query = executeStatement($query,$params);
	while($row=$query->fetch(PDO::FETCH_OBJ))
	{
		$list[$row->error_type]=$row->nbr_error;
	}
	return $list;
}

function countServiceAllState($dbconnection,$user=null,$ids=null)
{
	$list=array();
	$list['OK']=0;
	$list["warning"]=0;
	$list["critical"]=0;
	$list["unknown"]=0;
	$list["pending"]=0;
	if($user==null)
	{
		$query = $dbconnection->prepare("SELECT error_type,count(DISTINCT happen_id) as nbr
				FROM esia_errors
				JOIN esia_error_happen ON error_id = happen_error_id
				WHERE happen_acquitte=0
				GROUP BY error_type
				UNION
				SELECT 'OK',count(*)
				FROM esia_services
				WHERE service_previous_state = 0
				UNION
				SELECT 'pending',count(*)
				FROM esia_services
				WHERE service_last_stdout IS NULL AND service_previous_state = 4;");
		$query = executeStatement($query);
	}
	else
	{
		if(is_array($ids))
		{
			foreach($ids as $id)
			{
				if(!is_numeric($id))
					throw new EsiaDbException('$ids must be numeric');
			}
			$arrids =  implode(",",$ids);
			$query= $dbconnection->prepare("SELECT error_type,count(DISTINCT happen_id) as nbr
					FROM esia_errors
					JOIN esia_error_happen ON error_id = happen_error_id
					WHERE happen_acquitte=0 
						AND happen_device_id IN ( SELECT * FROM getUserAccessNode(:user))
						AND happen_service_id IN ($arrids)
					GROUP BY error_type
					UNION
					SELECT 'OK',count(*)
					FROM esia_services
					WHERE service_previous_state = 0 
						AND service_device_id IN ( SELECT * FROM getUserAccessNode(:user))
						AND service_id IN ($arrids)
					UNION
					SELECT 'pending',count(*)
					FROM esia_services
					WHERE service_last_stdout IS NULL 
						AND service_previous_state = 4 
						AND service_device_id IN ( SELECT * FROM getUserAccessNode(:user))
						AND service_id IN ($arrids);");
			$query = executeStatement($query,array(':user'=>$user));				
		}
		else
		{
			$query= $dbconnection->prepare("SELECT error_type,count(DISTINCT happen_id) as nbr
					FROM esia_errors
					JOIN esia_error_happen ON error_id = happen_error_id
					WHERE happen_acquitte=0 AND happen_device_id IN ( SELECT * FROM getUserAccessNode(:user))
					GROUP BY error_type
					UNION
					SELECT 'OK',count(*)
					FROM esia_services
					JOIN esia_device ON service_device_id = device_id
					WHERE service_previous_state = 0 AND service_device_id IN ( SELECT * FROM getUserAccessNode(:user))
					UNION
					SELECT 'pending',count(*)
					FROM esia_services
					WHERE service_last_stdout IS NULL AND service_previous_state = 4 AND service_device_id IN ( SELECT * FROM getUserAccessNode(:user));");
			$query = executeStatement($query,array(':user'=>$user));
		}

	

	}
	while($row=$query->fetch(PDO::FETCH_OBJ))
	{

		$list[$row->error_type]=$row->nbr;
	}
	return $list;
}

function getServices($dbconnection,$node=null,$service=null)
{
	$sql="SELECT plugin_id,plugin_name,plugin_path,plugin_type,plugin_trap_oid,plugin_description,
	service_id,service_name,service_interval_time,service_parameter,service_previous_state,
	to_char(service_last_exec_time,'DD-MM-YYYY HH24:MI:SS') as last_exec_time,service_last_stdout,to_char(service_time_added,'DD-MM-YYYY HH24:MI:SS') as check_time,
	device_id,device_name,device_type,
	program_id, program_name,program_path, response_parameter,response_ok, response_warning, response_critical, response_unknown,service_priority
		
	FROM esia_plugins
	JOIN esia_services ON plugin_id = service_plugin_id
	JOIN esia_device ON service_device_id = device_id
	LEFT JOIN esia_service_response_program ON service_id = response_service_id
	LEFT JOIN esia_program  ON program_id = response_program_id";
	$plugin = array();
	$ret;
	$params =array();
	if(!empty($service))
	{
		if(is_numeric($service))
		{
			$query= $dbconnection->prepare($sql." WHERE service_id=? AND service_delete=FALSE;");
			$params[]=$service;
		}
		elseif(is_array($service))
		{
			foreach($service as $id)
			{
				if(!is_numeric($id))
					throw new EsiaDbException('$service array must be numeric');
			}
			$arrids = implode(',', $service);
			$query= $dbconnection->prepare($sql." WHERE service_id IN ($arrids) AND service_delete=FALSE;");
		}
		else
		{
			throw new EsiaDbException('$service must be numeric');
		}
	}
	elseif(!empty($node))
	{
		if(is_numeric($node))
			$query= $dbconnection->prepare($sql." WHERE device_id=? AND service_delete=FALSE ORDER BY service_name;");
		elseif(is_string($node))
			$query= $dbconnection->prepare($sql." WHERE device_name=? AND service_delete=FALSE ORDER BY service_name ;");
		$params[]=$node;
	}
	else
	{
		throw new EsiaDbException('$node must not be null');
	}
	$query = executeStatement($query,$params);
	$q_isack = $dbconnection->prepare("SELECT happen_service_id FROM esia_error_happen WHERE happen_acquitte=0 AND happen_service_id=?");
	$q_rrd = $dbconnection->prepare("SELECT rrd_type,rrd_regex,rrd_time,rrd_name,rrd_max,rrd_min,rrd_graph_num,rrd_calc FROM rrd_plugins WHERE rrd_plugin_id=?;");
	while($row=$query->fetch(PDO::FETCH_OBJ))
	{
		$status = $row->service_previous_state;
		if($status!=0 && $status!=4)
		{
			$qe_status = executeStatement($q_isack,array($row->service_id));
			if($qe_status->rowCount()==0)
			{
				$status =0; 
			}
		}
		$rrd = executeStatement($q_rrd,array($row->plugin_id));
		$value=array();
		while($rowrrd=$rrd->fetch(PDO::FETCH_OBJ))
		{
			if(preg_match($rowrrd->rrd_regex,$row->service_last_stdout,$regs))
			{
				$value[$rowrrd->rrd_name]=$regs[1];
			}
		}
		$rrd = executeStatement($q_rrd,array($row->plugin_id));
		$value=array();
		while($rowrrd=$rrd->fetch(PDO::FETCH_OBJ))
		{
			if(preg_match($rowrrd->rrd_regex,$row->service_last_stdout,$regs))
			{
				$value[$rowrrd->rrd_name]=$regs[1];
			}
		}
		$plugin[]=array('id'=>$row->service_id,
				'name'=>$row->service_name,
				'plugin_id'=>$row->plugin_id,					
				'path'=>$row->plugin_path,
				'type'=>$row->plugin_type,
				'trap_oid'=>$row->plugin_trap_oid,
				'parameter'=>$row->service_parameter,
				'priority'=>$row->service_priority,
				'state'=>$status,
				'interval_time'=>$row->service_interval_time,
				'last_exec_time'=>$row->last_exec_time,
				'added_time'=>$row->check_time,
				'stdout'=>$row->service_last_stdout,
				'description'=>$row->plugin_description,
				'check_plugin_name'=>$row->plugin_name,
				'device_name'=>$row->device_name,
				'device_id'=>$row->device_id,
				'device_type'=>$row->device_type,
				'alerting_program_id'=>$row->program_id,
				'alerting_program_name'=>$row->program_name,
				'alerting_program_path'=>$row->program_path,
				'alerting_program_parameter'=>$row->response_parameter,
				'alerting_program_ok'=>$row->response_ok,
				'alerting_program_warning'=>$row->response_warning,
				'alerting_program_critical'=>$row->response_critical,
				'alerting_program_unknown'=>$row->response_unknown,
				'alerting_program_ok'=>$row->response_ok,
				'real_state'=>$row->service_previous_state,
				'value'=>$value
		);
	}

	return $plugin;
}

function getGroupNodeStates($dbconnection,$user,$group,$child=true)
{
	if(is_numeric($group))
	{
		if($child===false)
		{
			$sql ='SELECT SUM(warning) as warning,SUM(critical) as critical,SUM(unknown) as unknown,SUM(ok) as OK
					FROM esia_device_group
					LEFT JOIN esia_device_group_in ON device_group_id = group_in_device_group_id
					LEFT JOIN view_node_state AS T1 ON T1.device_id = group_in_device_id
					WHERE T1.device_id IN ( SELECT * FROM getUserAccessNode(:userid)) 
						AND group_in_device_group_id IN (SELECT group_id FROM getAllChildDeviceGroup(:groupid)) 
					GROUP BY device_group_id,device_group_parent
                                            ORDER BY device_group_parent DESC';
		}
		else
		{
			$sql ='SELECT device_group_id,device_group_name,device_group_parent,SUM(warning) as warning,SUM(critical)  as critical,SUM(unknown) as unknown,SUM(ok) as ok 
					FROM esia_device_group
					LEFT JOIN esia_device_group_in ON device_group_id = group_in_device_group_id
					LEFT JOIN view_node_state AS T1 ON T1.device_id = group_in_device_id

					WHERE (T1.device_id IN ( SELECT * FROM getUserAccessNode(:userid)) OR T1.device_id IS NULL)
						AND device_group_id IN (SELECT group_id FROM getAllChildDeviceGroup(:groupid))
					GROUP BY device_group_id,device_group_parent
					ORDER BY device_group_parent DESC';
		}
		$query = $dbconnection->prepare($sql);
		$query = executeStatement($query,array('userid'=>$user,'groupid'=>$group));
	}
	else
	{
			$sql ='SELECT device_group_id,device_group_name,device_group_parent,SUM(warning) as warning,SUM(critical)  as critical,SUM(unknown) as unknown,SUM(ok) as ok 
					FROM esia_device_group
					LEFT JOIN esia_device_group_in ON device_group_id = group_in_device_group_id
					LEFT JOIN view_node_state AS T1 ON T1.device_id = group_in_device_id

					WHERE (T1.device_id IN ( SELECT * FROM getUserAccessNode(:userid)) OR T1.device_id IS NULL)
						AND device_group_id IN (SELECT group_id FROM getAllChildDeviceGroup(NULL))
					GROUP BY device_group_id,device_group_parent
					ORDER BY device_group_parent DESC';
		$query = $dbconnection->prepare($sql);
		$query = executeStatement($query,array('userid'=>$user));
	}
	//Model_esia::countNodeState($dbconnection,$id);
	
	$arr = array();
	$result = $query->fetchAll(PDO::FETCH_ASSOC);
	//on ne veux que l'état du groupe passé en paramètre
	if($child===false)
	{
		$row = $result[0];
		$arr[$group]=array('ok'=>$row['ok'],
								'warning'=>$row['warning'],
								'critical'=>$row['critical'],
								'unknown'=>$row['unknown']);
	}
	else
	{
		//on sélectionne les fils de premier niveau
		foreach($result as $row)
		{
			if(is_numeric($group))
			{
				//on prend les fils du parents
				if($row['device_group_parent'] == $group)
				{
					//on récuère l'état du sous fils.
					$state = returnGroupNodeState($result, $row['device_group_id']);
					$arr[] = array('id'=>$row['device_group_id'],
								'name'=>$row['device_group_name'],
								'parent'=>$row['device_group_parent'],
								'ok'=>$state['ok'],
								'warning'=>$state['warning'],
								'critical'=>$state['critical'],
								'unknown'=>$state['unknown'],
								'pending'=>$state['pending'],
								'child'=>$state["child"]	);

				}
			}
			else
			{
				if(empty($row['device_group_parent']))
				{
					$state = returnGroupNodeState($result, $row['device_group_id']);
					$arr[] = array('id'=>$row['device_group_id'],
								'name'=>$row['device_group_name'],
								'parent'=>$row['device_group_parent'],
								'ok'=>$state['ok'],
								'warning'=>$state['warning'],
								'critical'=>$state['critical'],
								'unknown'=>$state['unknown'],
								'pending'=>$state['pending'],
								'child'=>$state["child"]	);
				}
			}
		}	
	}		
	return $arr;
}
function returnGroupNodeState($table,$groupid)
{
	$list['ok']=0;
	$list["warning"]=0;
	$list["critical"]=0;
	$list["unknown"]=0;
	$list["pending"]=0;
	$list["child"] = null;
	$arr=null;
	
	foreach($table as $row)
	{
		if($row['device_group_parent'] == $groupid)
		{
			$state = returnGroupNodeState($table, $row['device_group_id']);
	
			$arr[] = array('id'=>$row['device_group_id'],
					'name'=>$row['device_group_name'],
					'parent'=>$row['device_group_parent'],
					'ok'=>$state['ok'],
					'warning'=>$state['warning'],
					'critical'=>$state['critical'],
					'unknown'=>$state['unknown'],
					//	'pending'=>$row['pending'] + $state['pending'],
					'child'=>$state["child"]	);
			$list['ok'] += $state['ok'];
			$list['warning'] += $state['warning'];
			$list['critical'] += $state['critical'];
			$list['unknown'] += $state['unknown'];
			//		$list['pending'] += $state['pending'] + $row['pending'];
		}
		elseif($row['device_group_id'] == $groupid)
		{
			$list['ok'] += $row['ok'];
			$list['warning'] += $row['warning'];
			$list['critical'] += $row['critical'];
			$list['unknown'] += $row['unknown'];
			//	$list['pending'] += $row['pending'];
		}
	}
	
	$list["child"]=$arr;
	return $list;
}

function getGroupName($dbconnection,$group)
{
	$sql = "SELECT device_group_name FROM esia_device_group WHERE device_group_id=?";
	$param=array($group);
	$query= $dbconnection->prepare($sql);
	$query = executeStatement($query,$param);
	$row=$query->fetch(PDO::FETCH_OBJ);

	return $row->device_group_name;
}

?>
