#!/usr/bin/perl -w
############################## check_mail #################
my $Version='3.0';
# Date : Jun 05 2017
# Author  : Nathan Faes
# Help : esia-sa.com
#################################################################

use strict;
use Switch;
use DateTime;
use DateTime::Format::Strptime;
use Text::Trim;
use Getopt::Long;
use Mail::IMAPClient;
use Mail::POP3Client;
use IO::Socket::SSL;
use MIME::Base64;
use Email::MIME;
use Encode;
use HTML::Entities;

my $PATH='/tmp/';
my $PROG=$0;

my $help;

my $hostname;

my $port;
my $ssl;
my $method;

my $user;
my $password;
#my $authfile="/usr/local/esia/check_mail_auth.conf";
my $authfile="/etc/esia/mail.conf";

my $from;
my $number;
my $days;
my $hours;

my $subject;
my $content;
my $unknown;

my $ok;
my $warning;
my $critical;

my $directory;

my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3);
my %CODES=reverse %ERRORS;
my @PRIORITIES=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK}
);

sub priority_state{
        my($a,$b) = @_;

        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@PRIORITIES){
                if ( $v == $a || $v == $b ) { return $v; }
        }
	return -1;
}

sub exit_with_msg{
	my($msg,$state)=@_;
	unless ( defined $state) { $state=$ERRORS{UNKNOWN}; }
	print $CODES{$state}," - ",$msg,"\n";	
	exit $state;
}

sub check{
	my($text,$regex,$regexType,$actualState)=@_;

	if( defined($regex) and $text =~ m/$regex/ ){
		#print"Match : $1\n";
		return $regexType unless $actualState != -1;
		return priority_state($regexType,$actualState);	
	}
	return priority_state(-1,$actualState);
}

sub print_help(){
 	printf "Use IMAP/POP to read mails into a box and apply ok, warning and/or critical regex.\n";
        print "\n";
        print_usage();
        print "\n";
        print " -H (--hostname)   MailboxHostname to query - (required)\n";

	print " -p (--port)       Mailbox port\n";
        print " -s (--ssl)        SSL/TLS authentication (default none)\n";
	print " -m (--method)     The method/protocol (default imap)\n";

	print " -U (--user)       User login\n";
	print " -P (--password)   User password\n";
	print " -A (--authfile)   Authentication file\n";

	print " -f (--from)       Email sender part (default all)\n";
	print " -n (--number)     Analyse the n last mails (default to 1 if date os not set)\n";
	print " -d (--days)       Number of days";
	print " -D (--directory)  Directory (default : inbox)";
	print " -t (--hours)      Number of hours";

	print " -S (--subject)    Analyse the subject\n";
	print " -C (--content)    Analyse the content (default)\n";
	
	print " -o (--ok)         Regex for ok\n";
	print " -w (--warning)    Regex for warning\n";
	print " -c (--critical)   Regex for critical\n";

	print " -u (--unknown)   Get an unknwown error when no match found\n";

        print " -h (--help)       Usage help\n";
	print "\n";
}

sub print_usage(){
	print "Usage: \n";
	print " $PROG -H <HOSTNAME> [-p <PORT>] [ -s ] -U <USER> -P <PASSWORD> [ -f <MAIL> ] [ -n <NUMBER>] [ -d <NUMBER> ] [ -t <NUMBER> ] [ -S ] [ -C ] [ -o <REGEX> ] [ -w <REGEX> ] [ -c <CRITICAL> ]\n" ;
	print " $PROG -H <HOSTNAME> [-p <PORT>] [ -s ] -A <FILE> [ -f <MAIL> ] [ -n <NUMBER>] [ -d <NUMBER> ] [ -S ] [ -C ] [ -o <REGEX> ] [ -w <REGEX> ] [ -c <CRITICAL> ]\n";
	print " $PROG -h | --help \n";

}

sub pop3{
  	my($host,$port,$ssl,$user,$pass,$number,$since,$from)=@_;
        my @ret;

	#configuration
      	my $filter="";
        if ( defined $from      )       { $filter="$filter from $from";                         }
        if ( defined $since      )       { $filter="$filter since ".$since->strftime("%d-%h-%Y"); }
        if ( length $filter == 0)       { $filter="all";    }
        if( defined $number) { print "POP3 - FILTER : $filter, MAX : $number\n"; }
        else { print "POP3 - FILTER : $filter, MAX : $number\n"; }

	unless ( defined $port) {
                if( $ssl )      { $port=995; }
                else            { $port=110; }
        }

	#ouverture de connection
	my $pop =  new Mail::POP3Client( 
                HOST     => "$host:$port",
		USER     => $user,
                PASSWORD => $pass,
		USESSL	 => $ssl,
        ) or exit_with_msg "Cannot connect to $host as $user: $@";

        $pop->connect or exit_with_msg "Could not connect: $@\n";

	#Récupération des emails
	my $last = $pop->POPStat or exit_with_msg "Impossible to get last mail: $@\n";
	my $first=0;
	if( defined $number ) { $first=$last-$number; }
	if( $first < 0 ) { $first=0; }

	for(my $i = $last; $i > $first ; $i-- ) {
		my %header;
		foreach ( $pop->Head( $i ) ) {
      			if( /^(From|Subject|Date):\s+(.*)$/i ) {
				$header{$1}=trim($2);
			}
    		}

		if( defined $since ){
			my $parser=DateTime::Format::Strptime->new( pattern => "%d %b %Y %H:%M:%S %z");
			my $date=$parser->parse_datetime( $header{Date} );

			if( !defined($date)) {
  			 	$parser=DateTime::Format::Strptime->new( pattern => "%d %b %Y %H:%M:%S %Z");
               		 	$date=$parser->parse_datetime( $header{Date} );
			}

			if( DateTime->compare($date,$since) < 0 ) { return @ret }
		}
	
		if( !defined($from) or $header{From} =~ /.*$from.*/i ) {
			push(@ret,{
				"from" 		=> $header{From},
				"subject" 	=> $header{Subject},
				"date"		=> $header{Date},
				"content"	=> join("\n",$pop->Body($i)),
			});
		}
		else {
			if( $first > 0 ) { $first--; }
		}
        }

	#fermeture
        $pop->close;
	return @ret;
}

sub imap{
	my $box="inbox";
	my($host,$port,$ssl,$user,$pass,$number,$date,$dateUTC,$from)=@_;
	my @ret;

	#configuration
	my $filter="";
	if ( defined $from      )       { $filter="$filter from $from"; 			}
	if ( defined $date      )       { $filter="$filter since ".$date->strftime("%d-%h-%Y");	}
	if ( length $filter == 0)	{ $filter="all";					}

	my $filterToShow = $date->add(days => 1)->strftime("%d-%h-%Y");
	if( defined $number) { print "IMAP - FILTER : $filterToShow, MAX : $number\n"; }
	else { print "IMAP - FILTER : $filterToShow, MAX : N/A\n"; } 

	if( defined $directory) { $box = $box."/".$directory }

	unless ( defined $port) {
		if( $ssl ) 	{ $port=993; }
		else 		{ $port=143; }
	}

	#ouverture de connection
	my $imap;
	if( defined $ssl ){
		my $socket=new IO::Socket::SSL("$host:$port") or exit_with_msg "Cannot create SSL socket $host:$port : $@";
     		$imap = Mail::IMAPClient->new(
                        Socket          => $socket,
                        User            => $user,
                        Password        => $pass,
                ) or exit_with_msg "Cannot connect to $host as $user: $@";
	}
	else {
		$imap = Mail::IMAPClient->new(
			Server		=> "$host:$port",
			User   		=> $user,
			Password	=> $pass,
		) or exit_with_msg "Cannot connect to $host as $user: $@";
	}

	#Récupération des emails
	$imap->select($box) or exit_with_msg "Could not select: $@\n";

	$filter =~ s/^\s+//; #Supprime les espaces en début de chaine pour le filtre
	my @finded=$imap->search($filter) or warn "search: None found\n";
	if ( !defined($number) or scalar @finded < $number ) { $number = scalar @finded; }
	my @ids;
	for(my $i = $number; $i > 0 ; $i--){
		push(@ids,pop(@finded));
	}

    	foreach my $id (@ids)
	{
		if( defined $subject )
            	{
                        my $parser=DateTime::Format::Strptime->new( pattern => "%d %b %Y %H:%M:%S %z");
                        my $dateCompared=$parser->parse_datetime( $imap->date($id) );
			if(defined $dateCompared && $dateCompared gt $date)
                        {
                        	push(@ret,{
                        	        "subject" => $imap->subject($id),
                	                "date"    => $imap->date($id),
        	                        "from"    => $imap->get_header($id,"From")
	                        });
                        }
            	}
            	else
            	{
                        my $parser=DateTime::Format::Strptime->new( pattern => "%d %b %Y %H:%M:%S %z");
                        my $dateCompared=$parser->parse_datetime( $imap->date($id) );
			if(defined $dateCompared && $dateCompared gt $date)
                        {
                                push(@ret,{
                                        "subject" => $imap->subject($id),
                         		"content" => $imap->body_string($id),
			                "date"    => $imap->date($id),
                                        "from"    => $imap->get_header($id,"From")
                                });
                        }
            	}
    	}
	#fermeture	
	$imap->close;
	return @ret;
}

#Récupération des paramètres
&Getopt::Long::config('bundling');
GetOptions(
	"H=s"	=> \$hostname,	"hostname=s"	=> \$hostname,
	"p=i" 	=> \$port,	"port=i"	=> \$port,
	"s"	=> \$ssl,	"ssl"		=> \$ssl,
	"m=s"	=> \$method,	"method=s"	=> \$method,
	"U=s"	=> \$user,	"user=s"	=> \$user,
	"P=s"	=> \$password,	"password=s"	=> \$password,
	"A=s"	=> \$authfile,	"authfile=s"	=> \$authfile,
	"f=s"	=> \$from,	"from=s"	=> \$from,
	"n=i"	=> \$number,	"number=i"	=> \$number,
	"d=i"	=> \$days,	"days=i"	=> \$days,
	"D=s"	=> \$directory,	"directory=s"	=> \$directory,
	"S"	=> \$subject,	"subject"	=> \$subject,
	"C"	=> \$content,	"content"	=> \$content,
	"u:s"	=> \$unknown,	"unknown:s"	=> \$unknown,
	"o=s"	=> \$ok,	"ok=s"		=> \$ok,
	"w=s"	=> \$warning,	"warning=s"	=> \$warning,
	"c=s"	=> \$critical,	"critical=s"	=> \$critical,
	"t=i"	=> \$hours,	"hours=i"	=> \$hours,
	"h"	=> \$help,	"help"		=> \$help
);

if ( $help ) { 
	print_help(); 
	exit $ERRORS{'OK'};
}

#authentication file
if ( defined $authfile ){
	$authfile =~ tr/>//d; #Suppression du symbole > qui pourrait permettre d'écraser le fichier de config
	open(FILEHANDLER,$authfile) or exit_with_msg "Could not open file $authfile : $@\n";

	while(<FILEHANDLER>){
		$hostname=$1 	if ( /^\s*hostname\s*:\s*(\S+)\s*$/i and !defined $hostname );
		$port=$1 	if ( /^\s*port\s*:\s*(\S+)\s*$/i and !defined $port );
		$user=$1 	if ( /^\s*user\s*:\s*(\S+)\s*$/i and  !defined $user);
		$password=$1 	if ( /^\s*password\s*:\s*(\S+)\s*$/i and !defined $password );
	}

	close(FILEHANDLER);
}

#Filter config
my $date;
my $dateUTC = DateTime->now();
$date = DateTime->now();
$date->set_time_zone('Europe/Madrid');
my $newDay = $days + 1;
my $utcHour = (24-$date->strftime("%H"));

$date = $date->subtract( days => $newDay );
$dateUTC->subtract( days => $newDay );

if( defined $hours ) { 
	$date = $date->subtract( hours => $hours );
	$dateUTC = $dateUTC->subtract( hours => $hours );
}

if(!defined $days && !defined $hours){
	unless( defined $number ) { $number=1; }
}
if( defined $directory) {print "Directory : $directory\n";}
else {print "Directory : INBOX\n";}

#récupération des emails
my @mails;
if( defined $method ){
	switch($method){
		case /pop/i 	{   @mails=pop3($hostname,$port,$ssl,$user,$password,$number,$date,$from); }
		case /imap/i	{   @mails=imap($hostname,$port,$ssl,$user,$password,$number,$date,$dateUTC,$from); }
		else 		{  exit_with_msg( "The method/protocol [ $method ] is unknown" ); }
	}
}
else 	{   @mails=imap($hostname,$port,$ssl,$user,$password,$number,$date,$dateUTC,$from);}

#analyse
unless ( defined( $subject ) or defined ( $content )) { $content=1; }

my $co=0;
my $cw=0;
my $cc=0;
my $cu=0;
my $nm=0;

my $mainstate=$ERRORS{OK};
my $lmcm; #last most critical mail
foreach my $mail (@mails){
	my $state= -1;

	if( defined $subject ) {
	$state=check(encode('UTF-8', decode('MIME-Header',$mail->{subject})),$ok,$ERRORS{OK},$state);
        $state=check(encode('UTF-8', decode('MIME-Header',$mail->{subject})),$warning,$ERRORS{WARNING},$state);
        $state=check(encode('UTF-8', decode('MIME-Header',$mail->{subject})),$critical,$ERRORS{CRITICAL},$state);
	} 

	if( defined $content ) {
	#	if( $mail->{content} =~ m/^\s*([A-Za-z0-9+\/]{4})*([A-Za-z0-9+\/]{3}=|[A-Za-z0-9+\/]{2}==)?\s*$/mg ) {
	#		my $decoded = decode_base64($mail->{content});
	#		$mail->{content} = $decoded;
	#	}
		my $decoded = decode_base64($mail->{content});
		$mail->{content} .= $decoded;

		if($mail->{content} =~ m/Content-Transfer-Encoding: base64/)
		{
			my $parsed = Email::MIME->new($mail->{content});
			my @parts = $parsed->parts; # These will be Email::MIME objects, too.
			my $i=0;
			for my $part (@parts) { 
				$i++;
			    my $content_type = $parsed->content_type;
			    if ($content_type eq "text/plain") {
			         #print $part->body ();
			    }
			    elsif ($content_type =~ m/text\/html/) {
					$mail->{content} = decode_entities($part->body());
			    }  
			       #$mail->{content} = $part->body();   
			    # Handle some more cases here
			}
		}
		#print "CONTENU : ".encode('UTF-8', decode('MIME-Header',$mail->{content}))."\n";
        $state=check(encode('UTF-8', decode('MIME-Header',$mail->{content})),$ok,$ERRORS{OK},$state);
        $state=check(encode('UTF-8', decode('MIME-Header',$mail->{content})),$warning,$ERRORS{WARNING},$state);
        $state=check(encode('UTF-8', decode('MIME-Header',$mail->{content})),$critical,$ERRORS{CRITICAL},$state);
	}

	switch($state){
		case ($ERRORS{OK}) 	{ $co++; }
                case ($ERRORS{WARNING})	{ $cw++; }
                case ($ERRORS{CRITICAL}){ $cc++; }
                case ($ERRORS{UNKNOWN}) { $cu++; }
		case -1			{ $nm++; }  #Alors aucun check n'a matcher de regex.
	}

	my $oldstate=$mainstate;
	$mainstate=priority_state($mainstate,$state);
	if( $oldstate != $mainstate ){
		$lmcm=$mail; #last most critical mail
	}
}

#Résultat 
my $total=scalar @mails;
print "Mails : $total, UNMATCHED : $nm, OK : $co, WARNING : $cw, CRITICAL : $cc, UNKNOWN : $cu\n";

if( $total == ( $nm + $co ) ){
	print "--------------------------------------------------\n";
	if( defined $unknown && $co == 0 ) {
		if($unknown eq "critical")
		{
			print "CRITICAL : - no alert received\n";
			$mainstate = $ERRORS{CRITICAL};
		}
		elsif($unknown eq "warning")
		{
			print "WARNING : - no alert received\n";
			$mainstate = $ERRORS{WARNING};
		}
		elsif($unknown eq "ok")
		{
			print "OK : - no alert received\n";
			$mainstate = $ERRORS{OK};
		}
		else
		{
			print "UNKNOWN : - no alert received\n";
			$mainstate = $ERRORS{UNKNOWN};
		}
	}
	else{
		print $CODES{$mainstate}, " - All is right\n";
	}
}
else{
	if (!defined($lmcm->{subject}) || $lmcm->{subject} eq "")
	{
		print "Last most critical =>\n DATE : ",$lmcm->{date}, "\n FROM : ", $lmcm->{from}, "\n<=";
	}
	else
	{
		print "Last most critical =>\n DATE : ",$lmcm->{date}, "\n FROM : ", $lmcm->{from}, "\n OBJ : ",encode('UTF-8', decode('MIME-Header',$lmcm->{subject})),"\n<=";
	}
	print "------------------------------------------------\n";
	print $CODES{$mainstate}, " - ", $total - $nm ," alerts received.\n";
}

exit $mainstate;
