# --- fichier SNMP.pm ---
package ESIA::SNMP;
use strict;
use warnings;
use Net::SNMP;
use Getopt::Long qw(GetOptionsFromArray);

sub Session
{
    my @myopts = @_;
    Getopt::Long::Configure( "pass_through" );
    my ($v3_username,$v3_authkey,$v3_privkey);
    my $v3_protocols='md5';
    my $snmp_version = 2;
    my $timeout = 2;
    my $community= 'public';
    my $hostname= '';
    my $snmp_domain ="UDP/IPV4";
    my $retries =2;
    my $ret = GetOptionsFromArray(\@myopts,"C=s" => \$community,	"community=s" => \$community,
	                                        "H=s" => \$hostname,	"hostname=s" => \$hostname,
	                                        "v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	                                        "t=i" => \$timeout,"timeout=i" => \$timeout,
                                            "U=s" => \$v3_username,"user=s" => \$v3_username,
                                            "A=s" => \$v3_authkey,"authkey=s" => \$v3_authkey,
                                            "P=s" => \$v3_privkey,"privkey=s" => \$v3_privkey,
                                            "L=s" => \$v3_protocols,"level=s" => \$v3_protocols,
                                            "r=i" => \$retries,"retries=i" => \$retries,
                                );
#              );                           
    my ($snmp, $snmperror);
    #on vérifie si on est en IPv6
    if($hostname =~ /(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))/ )
    {
        $snmp_domain ="UDP/IPV6";
    }

    if($snmp_version == 2)
    {
       $snmp_version='snmpv2c';
    }
    elsif($snmp_version == 1)
    {
       $snmp_version='snmpv1';
    }
    if(defined($v3_username))
    {
        if(!defined($v3_authkey))
        {
            $snmperror="No authentification key specified";
            return $snmp,$snmperror;
        }
        my $authprotocol=$v3_protocols;
        my $privrotocol;
        if($v3_protocols=~/([a-z0-5]*)\,([a-z0-5]*)/)
        {
            $authprotocol=$1;
            $privrotocol=$2;
        }
        if(defined($privrotocol))
        {
            if(!defined($v3_privkey))
            {
                $snmperror="No privacy key specified";
                return $snmp,$snmperror;
            }
            else
            {
                ($snmp, $snmperror) = Net::SNMP->session(
		            -hostname => $hostname,
		            -version => 'snmpv3',
                    -domain => $snmp_domain,
                    -timeout => $timeout,
		            -username => $v3_username,
                    -authkey =>$v3_authkey,
                    -authprotocol =>$authprotocol,
                    -privkey =>$v3_privkey,
                    -privprotocol =>$privrotocol,
	            );
            }
        }
        else
        {
            ($snmp, $snmperror) = Net::SNMP->session(
		        -hostname => $hostname,
		        -version => 'snmpv3',
                -domain => $snmp_domain,
                -timeout => $timeout,
		        -username => $v3_username,
                -authkey =>$v3_authkey,
                -authprotocol =>$authprotocol
	        );
        }
    }
    else 
    {
	    ($snmp, $snmperror) = Net::SNMP->session(
		    -hostname => $hostname,
		    -version => $snmp_version,
		    -community => $community,
            -domain => $snmp_domain,
            -timeout => $timeout,
	    );
    }
    return $snmp,$snmperror;
}
sub get
{
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}
sub getManufacturerOID
{
    my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";
    my($snmp) = @_;
	my $result = $snmp->get_request("$oid_sysObjectOID");
    if (!defined($result)) 
    {
	    return -1;
    }
    my $Sysoid_value = $$result{"$oid_sysObjectOID"};
    if($Sysoid_value =~/3\.6\.1\.4\.1\.([0-9]*)/)
    {
        $Sysoid_value = $1;
    }
	return $Sysoid_value;
}
sub usage
{
    my @used; 
    $used[0][0]='-H';
    $used[0][1]='--hostname';
    $used[0][2]= "Hostname to query - (required)";
    $used[1][0]='-C';
    $used[1][1]='--community';
    $used[1][2]= "SNMP read community (default: public)";
    $used[2][0]='-v';
    $used[2][1]='--snmp_version';
    $used[2][2]= "SNMP version [1|2] (default: 2)";
    $used[3][0]='-t';
    $used[3][1]='--timeout';
    $used[3][2]= "SNMP timeout (default: 2)";

    $used[4][0]='-r';
    $used[4][1]='--retries';
    $used[4][2]= "SNMP retries (default: 2)";

    $used[5][0]= "-U";
    $used[5][1]= "--username";
    $used[5][2]= "SNMP V3 user. Enable SNMP v3 request";

    $used[6][0]= "-A";
    $used[6][1]= "--authKey";
    $used[6][2]= "SNMP V3 authentification key. required if SNMPV3 is activated";
    $used[7][0]= "-P";
    $used[7][1]= "--privKey";
    $used[7][2]= "SNMP V3 private key";
    $used[8][0]= "-L";
    $used[8][1]= "--level";
    $used[8][2]= "SNMP V3 protocols [md5|sha],[aes|des] (default: sha)";


    return (\@used);
}
sub print_snmp_usage
{
    my $name = $_[0];
    my $addoutput = $_[1];
    print "Usage: \n";
	print "SNMPv1/v2c\n"; 
    print "  $name -H <HOSTNAME> -C <community> $addoutput\n";
	print "SNMPv3 authnopriv\n";
    print "  $name -H <HOSTNAME> -U <user> -L [md5|sha] -A <authentification key> $addoutput\n";
	print "SNMPv3 authpriv\n";
    print "  $name -H <HOSTNAME> -U <user> -L [md5|sha],[aes|des] -A <authentification key> -P <private key> $addoutput\n";
	print "  $name [-h | --help]\n";
	print "  $name [-V | --version]\n";
}
sub print_snmp_help
{
    my $name = $_[0];
    my $desc = $_[1];
    my $othervar =  $_[2];
    my $used = ESIA::SNMP::usage();
    ESIA::PLUGIN::print_header($name,$desc );
	print "\n";
    print "Plugin parameters: \n";
    printf ("     %-10s%-20s%s\n", 'short','long','description:');
    foreach my $key ( @$used) 
    {
        my @line = @$key; 
        printf ("     %-10s%-20s%s\n", $line[0],$line[1],$line[2]);
    }
    push(@$othervar,["-h","--help","print this help"]);
    push(@$othervar,["-V","--version","plugin version"]);
    foreach my $key ( @$othervar) 
    {
        my @line = @$key; 
        printf ("     %-10s%-20s%s\n", $line[0],$line[1],$line[2]);
    }
}
1;
